
/*
 * shared memory procedures
 *     
 * $Id: shm.c, 2000-06-09(Fri) 19:33:14 satosi-f $
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include <errno.h>

int make_new_pool(key_t shmkey, size_t pool_size)
{
  int chk;
  int flg;

  flg = 0666;
  flg = flg | IPC_CREAT;

  chk = shmget(shmkey, (int)pool_size, flg);
  if (chk < 0) {
    fprintf(stderr, "make_new_pool: fail shmget(): %s\n", sys_errlist[errno]);
    return chk;
  } 

  return chk;
}    

int delete_pool(int shmid)
{
  int chk;

  chk = shmctl(shmid, IPC_RMID, NULL);

  return chk;
}

int assign_pool(key_t shmkey)
{
  int chk;
  int flg;

  flg = 0666;
  
  chk = shmget(shmkey, (int)0, flg);
  if (chk < 0) {
    fprintf(stderr, "assign_pool: fail shmget(): %s\n", sys_errlist[errno]);
    return chk;
  }

  return chk;
}

int show_pool_stat(int shmid)
{
  struct shmid_ds act_buf, *buf;
  int chk;

  buf = &act_buf;

  fprintf(stdout, "====\n");
  fprintf(stdout, "ID %d\n", shmid);

  chk = shmctl(shmid, IPC_STAT, buf);
  if (chk < 0) {
    fprintf(stderr, "show_pool_stat: fail shmctl(): %s\n", sys_errlist[errno]);
    return;
  }
  
  fprintf(stdout, "---\n");
  fprintf(stdout, " key  = %d %#x\n", 
	  buf->shm_perm.key, buf->shm_perm.key);
  fprintf(stdout, " seq  = %d\n", buf->shm_perm.seq);
  fprintf(stdout, " mode = %#o\n", buf->shm_perm.mode);
  fprintf(stdout, " current uid/gid = %d/%d\n",
	  buf->shm_perm.uid, buf->shm_perm.gid);
  fprintf(stdout, " creator uid/gid = %d/%d\n",
	  buf->shm_perm.cuid, buf->shm_perm.cgid);
  fprintf(stdout, "---\n");

  fprintf(stdout, " segment size =%d %#x ( %d KB, %d MB)\n",
	  buf->shm_segsz, buf->shm_segsz,
	  buf->shm_segsz/1024, buf->shm_segsz/1048576); /* 1MB = 1048576 */

  fprintf(stdout, " last    pid = %d\n", buf->shm_lpid);
  fprintf(stdout, " creator pid = %d\n", buf->shm_cpid);

  fprintf(stdout, " n  attached process = %d\n", buf->shm_nattch);
#if 0
  fprintf(stdout, " cn attached process = %d\n", buf->shm_cnattach);
#endif
  fprintf(stdout, " time shmat/shmdt/change = %d/%d/%d\n",
	  buf->shm_atime, buf->shm_dtime, buf->shm_ctime);

  fflush(stdout);
}
    

  
