
/*
 * udprecv.c - receive udp packets and reassemble them.
 */

#include <stdio.h>
#include <sys/types.h>

#include <netinet/in.h>

#include <errno.h>

#include "sentry.h"

struct image_offset {
  int offset;
  int length;
  struct image_offset *next;
};

#define IBUF_SIZE       32
#define OLIST_SIZE      32
struct {
  int valid;                    /* valid flag (0:free, 1:used) */
  long long ts;                 /* timestamp */
  struct image_offset olist[OLIST_SIZE];
  caddr_t pimage;               /* pointer to image buffer */
  int size;                     /* image buffer size */
  int sum;                      /* summing of store */
} ibuf[IBUF_SIZE];


/*
 * image buffer list for udp packet
 */
void clear_ibuf(int id)
{
  ibuf[id].valid = 0;
  ibuf[id].size = 0;
  ibuf[id].sum = 0;
}

void init_ibuf()
{
  int ibuf_id;

  for(ibuf_id=0; ibuf_id<IBUF_SIZE; ibuf_id++)
    clear_ibuf(ibuf_id);
}

int find_ibuf(long long *t) 
{
  int ibuf_id;

  for(ibuf_id=0 ; ibuf_id<IBUF_SIZE; ibuf_id++)
    if ( ibuf[ibuf_id].valid == 1 && (*t) == ibuf[ibuf_id].ts )
      return ibuf_id;

  return -1;
}

int new_ibuf()
{
  int ibuf_id;

  for(ibuf_id=0; ibuf_id<IBUF_SIZE; ibuf_id++)
    if (ibuf[ibuf_id].valid == 0)
      return ibuf_id;

  fprintf(stderr, "fail to allocate new ibuf.\n");
  return -1;
}

int
join_ibuf(int id, int part_off, int part_len)
{
#if 0
  struct image_offset *olist = ibuf[id].olist;
  struct image_offset *p;
  struct image_offset *pos1, *pos2, *free;
  int i;

  if (part_off == 0) { /* top packet */
    olist->offset = part_off;
    for(i=1; i<OLIST_SIZE; i++)
      if (olist->offset + olist->length == olist[i].offset) {
	olist->length = part_len + olist[i].length;
	olist[i].offset = 0;
	olist[i].length = 0;
      }
  }
  else { /* not top packet */
    pos1 = pos2 = free = -1;
    for(i=0; i<OLIST_SIZE; i++) {
      if (olist[i].offset + olist[i].length == part_off)
	pos1 = i;
      if (part_off + part_len == olist[i].offset)
	pos2 = i;
      if (i > 0 && olist[i].offset == 0 && olist[i].length == 0)
	free = i;
      if (pos1 >= 0 && pos2 >= 0 && free > 0)
	break;
    }
  }

  /*** part2 ***/

  if (part_off == 0) { /* top packet */
    off_t->offset = part_off; /* = 0 */
    off_t->length = part_len;

    /* shrink */
    return;
  }

  /* part_off > 0 */

  /* search empty cell */
  pos = -1;
  for(i=1; i<OLIST_SIZE; i++)
    if (off_t[i].offset == 0 && off_t[i].length == 0)
      break;
  pos = i;

  /* shrink list */
  p = off_t;
  if (off_t->length == 0) {
    p = &off_t[1];
    prev = off_t;
  }
  for(; p; p->next) {
    if (part_off < p->offset) {
      if (part_off + part_length == p->offset) {
	p->offset = part_off;
	p->length += part_len;
      } else is (part_off + part_length < p->offset) {
	if (prev->offset = prev->length == part_off)
	  prev->length += part_len;
	else {
	  prev->next = &off_t[pos];
	  off_t[pos].offset = part_off;
	  off_t[pos].length = part_len;
	  off_t[pos].next = p;
	}
      }
    } else if (part_off == p->offset) {
      /* normally, nothring occured. */
    }
  }
#endif
}
  
int
udp_reassemble(sentry_image_header *h, char *data, int datalen, char **image) 
{
  int id;
  long long *tsp;
  int image_size, data_offset;

  tsp = (long long *)&h->hd_ts;

  if (ibuf[0].valid == 1 && (*tsp) == ibuf[0].ts) 
    id = 0;
  else {
    id = find_ibuf(tsp);
    if (id < 0)
      if (ibuf[0].valid == 0) 
	id = 0;
    else 
      id = new_ibuf(tsp);
    if (id < 0)
      return -1;

    /* dump ibuf */
    ibuf[id].valid = 1;
    ibuf[id].ts = *tsp;
  }

  image_size = ntohl(h->size);
  if (ibuf[id].size == 0 || ibuf[id].size < image_size) {

    fprintf(stdout, "new buffer %2d (new %ld, old %ld).\n", 
	    id, (long)image_size, (long)ibuf[id].size);

    if (ibuf[id].pimage != NULL) {
      fprintf(stdout, "resize buffer %2d.\n", id);

      free(ibuf[id].pimage);
      ibuf[id].size = 0;
    }
    ibuf[id].size = image_size;
    ibuf[id].pimage = (caddr_t)malloc(ibuf[id].size);
  }

  (*image) = ibuf[id].pimage;
  
  data_offset = ntohl(h->offset);
  if (data_offset + datalen > image_size) {
    fprintf(stderr, "Waring: offset is wrong.\n");
    datalen = image_size - data_offset;
  }

  /* 
   * store image: copy packets to stored.
   */
  bcopy(data, (char *)&(*image)[data_offset], datalen);
  ibuf[id].sum += datalen;

  join_ibuf(id, data_offset, datalen);

  if (ibuf[id].olist[0].offset == 0 && 
      ibuf[id].olist[0].length == h->size) 
    return ibuf[id].sum;
  else 
    return 0;
}
