/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *  
 *  $Id: $
 */

/*
 * image_converter.c - convert images
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>

#include "sentry.h"

#ifdef USE_CL
#include <dmedia/cl.h>
#endif

extern int errno;
extern int debug;

static int jpeg_system_initted = 0;

int
convert_none(char *cur, int size, int xsize, int ysize, int *nxsize, int *nysize, char **new)
{
  *new = cur;
  *nxsize = xsize;
  *nysize = ysize;
  return size;
}

int
init_jpeg_system(int x, int y)
{
  int chk;

  jpeg_system_initted = 1;

#ifndef USE_CL
  fprintf(stderr, "init_jpeg_system: not supported CL\n");
  return -1;
#else
  chk = clOpenComressor(CL_JPEG_COSMO, &compressorHdl);
  if (chk != SUCCESS) {
    fprintf(stderr, "init_jpeg_system: no COSMO system.\n");

    chk = clOpenCompressor(CL_JPEG_SOFTWARE, &compressorHdl);
    if (chk != SUCCESS) {
      fprintf(stderr, "init_jpeg_system: no JPEG library.\n");
      return -1;
    } 
    else
      fprintf(stderr, "init_jpeg_system: use jpeg software encoding.\n");
  } 
  else
    fprintf(stderr, "init_jpeg_system: have COSMO system.\n");

  clSetParam(compressorHdl, CL_IMAGE_WIDTH, x);
  clSetParam(compressorHdl, CL_IMAGE_HEIGHT, y);
  clSetParam(compressorHdl, CL_FORMAT, CL_FORMAT_XBGR);

  if (inner_buff != NULL) {
    free(inner_buff);
    inner_buff = NULL;
  }
  
  if ((inner_buff = malloc(x*y*5)) == NULL) {
    fprintf(stderr, "init_jpeg_system: no memory (%d)\n", errno);
    return -1;
  }
#endif /* USE_CL */

  return 0;
}

int
convert_jpeg(char *cur, int size, int xsize, int ysize, int *nxsize, int *nysize, char **new)
{
  int chk;
  int csize;

  if (jpeg_system_initted == 0) {
    if (init_jpeg_system(xsize, ysize)) {
      return -1;
    }
  }

#ifndef USE_CL
  fprintf(stderr, "convert_jpeg: not support CL\n");
  return -1;
#else
  chk = clOpenCompressor(CL_JPEG_COSMO, &compressorHdl);
  if (chk != SUCCESS) {
    fprintf(stderr, "init_jpeg_system: no COSMO system.\n");

    chk = clOpenCompressor(CL_JPEG_SOFTWARE, &compressorHdl);
    if (chk != SUCCESS) {
      fprintf(stderr, "init_jpeg_system: no JPEG library.\n");
      return -1;
    } else
      fprintf(stderr, "init_jpeg_system: use jpeg software encoding.\n");
  } else {
    fprintf(stderr, "init_jpeg_system: have COSMO system.\n");
  }

  clSetParam(compressorHdl, CL_IMAGE_WIDTH, x);
  clSetParam(compressorHdl, CL_IMAGE_HEIGHT, y);
  clSetParam(compressorHdl, CL_FORMAT, CL_FORMAT_XBGR);

  if (inner_buff != NULL) {
    free(inner_buff);
    inner_buff = NULL;
  }

  if ((inner_buff = malloc(x*y*5)) == NULL) {
    fprintf(stderr, "init_jpeg_system: no memory (%d)\n", errno);
    return -1;
  }
#endif /* USE_CL */

  return 0;
}
